# -*- coding: utf-8 -*-
"""Mixins for the django admin."""
# Django
from django.contrib import messages
from django.http import HttpResponseRedirect


class WorkflowAdminMixin(object):
    """
    A mixin to provide workflow transition actions.

    It will create an admin log entry.
    """

    change_form_template = 'transitions/change_form.html'

    def response_change(self, request, obj):
        """Add actions for the workflow events."""
        events = list(obj.get_available_events())
        for event in events:
            if '_' + event['transition'].name not in request.POST:
                continue

            before = obj.state
            if getattr(obj, event['transition'].name)():
                obj.save()
                after = obj.state
                message = ('Status changed from {0} to {1} by transition {2}'
                           .format(before, after, event['transition'].name))
                self.message_user(request, message, messages.SUCCESS)
                self.log_change(request, obj, message)
            else:
                message = ('Status could not be changed from '
                           '{0} by transition {1}'
                           .format(before, event['transition'].name))
                self.message_user(request, message, messages.ERROR)
            return HttpResponseRedirect('.')
        return super(WorkflowAdminMixin, self).response_change(request, obj)



# -*- coding: utf-8 -*-
"""Mixins for transition workflows."""

# Standard Library
from functools import partial

# 3rd-party
from transitions.extensions import MachineFactory


class StatusBase(object):
    """Base class for transitions and status definitions."""

    STATE_CHOICES = (
        # Override this!
        # https://docs.djangoproject.com/en/2.1/ref/models/fields/#choices
        # to provide human readable labels for states
        # (state, 'My Workflow state'),
    )

    TRANSITION_LABELS = {
        # Override this!
        # Provide human readable labels and css class for transitions
        # transition: {'label': 'Label', 'cssclass': 'default'}
    }

    SM_STATES = [
        # Override this!
        # list of available workflow states
    ]

    SM_INITIAL_STATE = None  # Initial state of the machine. Override this!

    SM_TRANSITIONS = [
        # Override this!
        # trigger, source, destination
    ]

    @classmethod
    def get_kwargs(cls):
        """Get the kwargs to initialize the state machine."""
        kwargs = {
            'initial': cls.SM_INITIAL_STATE,
            'states': cls.SM_STATES,
            'transitions': cls.SM_TRANSITIONS,
        }
        return kwargs


class StateMachineMixinBase(object):
    """
    Base class for state machine mixins.

    Class attributes:

    * ``status_class`` must provide ``TRANSITION_LABELS`` property
      and the ``get_kwargs`` class method (see ``StatusBase``).
    * ``machine`` is a transition machine e.g::

        machine = Machine(
            model=None,
            finalize_event='wf_finalize',
            auto_transitions=False,
            **status_class.get_kwargs()  # noqa: C815
        )

    The transition events of the machine will be added as methods to
    the mixin.
    """

    status_class = None  # Override this!
    machine = None  # Override this!

    def get_available_events(self):
        """
        Get available workflow transition events for the current state.

        Returns a dictionary:
            * ``transition``: transition event.
            * ``label``: human readable label for the event
            * ``cssclass``: css class that will be applied to the button
        """
        for trigger in self.machine.get_triggers(self.state):
            event = self.machine.events[trigger]
            ui_info = self.status_class.TRANSITION_LABELS[event.name]
            ui_info['transition'] = event
            yield ui_info

    def get_wf_graph(self):
        """Get the graph for this machine."""
        diagram_cls = MachineFactory.get_predefined(graph=True)
        machine = diagram_cls(
            model=self,
            auto_transitions=False,
            title=type(self).__name__,
            **self.status_class.get_kwargs()  # noqa: C815
        )
        return machine.get_graph()

    def __getattribute__(self, item):
        """Propagate events to the workflow state machine."""
        try:
            return super(StateMachineMixinBase, self).__getattribute__(item)
        except AttributeError:
            if item in self.machine.events:
                return partial(self.machine.events[item].trigger, self)
            raise



# -*- coding: utf-8 -*-
"""An opinonated example for a workflow mixin."""

# Django
from django.db import models


class WorkflowMigrationMixin(models.Model):
    """
    A mixin to provide workflow state and workflow date fields.

    This is a minimal example implementation.
    """

    class Meta:  # noqa: D106
        abstract = True

    wf_state = models.CharField(
        verbose_name='Workflow Status',
        null=True,
        blank=True,
        max_length=32,
        help_text='Workflow state',
    )

    wf_date = models.DateTimeField(
        verbose_name='Workflow Date',
        null=True,
        blank=True,
        help_text='Indicates when this workflowstate was entered.',
    )



import os
import sys

from setuptools import find_packages
from setuptools import setup

version = '0.2'

setup(name='django_transitions',
      version=version,
      description="Integrate pytransitions with Django.",
      long_description=(open("README.rst").read() + "\n"),
      classifiers=[
          'Framework :: Django',
          'Framework :: Django :: 1.11',
          'Framework :: Django :: 2.0',
          'Framework :: Django :: 2.1',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: BSD License',
      ],  # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='django permission restframework',
      author='Christian Ledermann',
      author_email='christian.ledermann@gmail.com',
      url='https://github.com/PrimarySite/django-transitions',
      license='BSD',
      packages=find_packages(exclude=['ez_setup', 'testproj', 'testapp']),
      package_dir={'django_transitions': 'django_transitions'},
      package_data={'django_transitions': ['templates/transitions/*.html']},
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          'Django',
          'transitions',
      ],
      entry_points="""
        # -*- Entry points: -*-
        """,
      )
